import _objectSpread from "@babel/runtime/helpers/objectSpread2";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _classCallCheck from "@babel/runtime/helpers/classCallCheck";
import _createClass from "@babel/runtime/helpers/createClass";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
import { decodePickingColor } from '@antv/l7-utils';
import { TileLayerManager } from "../manager/tileLayerManager";
import { debounce } from 'lodash';

var BaseTileLayer = /*#__PURE__*/function () {
  function BaseTileLayer(_ref) {
    var _parentSource$data;

    var parent = _ref.parent,
        rendererService = _ref.rendererService,
        mapService = _ref.mapService,
        layerService = _ref.layerService,
        pickingService = _ref.pickingService,
        transforms = _ref.transforms;

    _classCallCheck(this, BaseTileLayer);

    _defineProperty(this, "type", 'baseTile');

    _defineProperty(this, "initedTileset", false);

    _defineProperty(this, "pickColors", {
      select: null,
      active: null
    });

    _defineProperty(this, "viewchange", debounce(this.mapchange, 200));

    var parentSource = parent.getSource();

    var _ref2 = (parentSource === null || parentSource === void 0 ? void 0 : (_parentSource$data = parentSource.data) === null || _parentSource$data === void 0 ? void 0 : _parentSource$data.tilesetOptions) || {},
        sourceLayer = _ref2.sourceLayer;

    this.sourceLayer = sourceLayer;
    this.parent = parent;
    this.mapService = mapService;
    this.layerService = layerService;
    this.tileLayerManager = new TileLayerManager(parent, mapService, rendererService, pickingService, transforms);
    this.initTileSetManager();
    this.bindSubLayerEvent();
    this.bindSubLayerPick();
    this.scaleField = this.parent.getScaleOptions();
  }
  /**
   * 渲染瓦片的图层
   */


  _createClass(BaseTileLayer, [{
    key: "children",
    get: function get() {
      return this.tileLayerManager.children;
    }
  }, {
    key: "render",
    value: function render() {
      if (this.tileLayerManager) {
        this.tileLayerManager.render();
      }
    }
  }, {
    key: "clearPick",
    value: function clearPick(type) {
      if (type === 'mousemove') {
        this.tileLayerManager.tilePickManager.clearPick();
      }
    }
    /**
     * 清除 select 的选中状态
     */

  }, {
    key: "clearPickState",
    value: function clearPickState() {
      var _this = this;

      this.children.filter(function (child) {
        return child.inited && child.isVisible();
      }).filter(function (child) {
        return child.getCurrentSelectedId() !== null;
      }).map(function (child) {
        _this.selectFeature(child, new Uint8Array([0, 0, 0, 0]));

        child.setCurrentSelectedId(null);
      });
    }
    /**
     * 瓦片图层独立的拾取逻辑
     * @param target
     * @returns
     */

  }, {
    key: "pickLayers",
    value: function pickLayers(target) {
      return this.tileLayerManager.pickLayers(target);
    } // eslint-disable-next-line @typescript-eslint/no-unused-vars

  }, {
    key: "tileLoaded",
    value: function tileLoaded(tile) {//
    }
  }, {
    key: "tileError",
    value: function tileError(error) {
      console.warn('error:', error);
    }
  }, {
    key: "tileUnLoad",
    value: function tileUnLoad(tile) {
      this.tileLayerManager.removeChilds(tile.layerIDList, false);
    }
  }, {
    key: "tileUpdate",
    value: function tileUpdate() {
      // Base Function
      if (!this.tilesetManager) {
        return;
      }

      if (this.tilesetManager.isLoaded) {
        // 将事件抛出，图层上可以使用瓦片
        this.parent.emit('tiles-loaded', this.tilesetManager.currentTiles);
      }
    }
  }, {
    key: "setPickState",
    value: function setPickState(layers) {
      var _this2 = this;

      if (this.pickColors.select) {
        var selectedId = decodePickingColor(this.pickColors.select);
        layers.map(function (layer) {
          _this2.selectFeature(layer, _this2.pickColors.select);

          layer.setCurrentSelectedId(selectedId);
        });
      }

      if (this.pickColors.active) {
        var _selectedId = decodePickingColor(this.pickColors.active);

        layers.filter(function (layer) {
          return layer.inited && layer.isVisible();
        }).map(function (layer) {
          layer.hooks.beforeHighlight.call(_this2.pickColors.active);
          layer.setCurrentPickId(_selectedId);
        });
      }
    }
  }, {
    key: "bindSubLayerPick",
    value: function bindSubLayerPick() {
      var _this3 = this;

      this.tileLayerManager.tilePickManager.on('pick', function (e) {
        // @ts-ignore
        var _e$pickedColors = _slicedToArray(e.pickedColors, 3),
            r = _e$pickedColors[0],
            g = _e$pickedColors[1],
            b = _e$pickedColors[2];

        if (e.type === 'click') {
          var restLayers = _this3.children.filter(function (child) {
            return child.inited && child.isVisible() && child.isVector;
          }).filter(function (child) {
            return child !== e.layer;
          });

          _this3.setSelect(restLayers, [r, g, b]);
        } else {
          _this3.setHighlight([r, g, b]);
        }
      });
      this.tileLayerManager.tilePickManager.on('unpick', function () {
        _this3.pickColors.active = null;
      });
    }
  }, {
    key: "setHighlight",
    value: function setHighlight(pickedColors) {
      var pickId = decodePickingColor(pickedColors);
      this.pickColors.active = pickedColors;
      this.children.filter(function (child) {
        return child.inited && child.isVisible() && child.isVector;
      }) // Tip: 使用 vectorLayer 上的 pickID 优化高亮操作（过滤重复操作）
      // @ts-ignore
      .filter(function (child) {
        return child.getPickID() !== pickId;
      }).map(function (child) {
        // @ts-ignore
        child.setPickID(pickId);
        child.hooks.beforeHighlight.call(pickedColors);
      });
    }
  }, {
    key: "setSelect",
    value: function setSelect(layers, pickedColors) {
      var _this4 = this;

      var selectedId = decodePickingColor(pickedColors);
      layers.map(function (layer) {
        if (layer.getCurrentSelectedId() === null || selectedId !== layer.getCurrentSelectedId()) {
          _this4.selectFeature(layer, pickedColors);

          layer.setCurrentSelectedId(selectedId);
          _this4.pickColors.select = pickedColors;
        } else {
          _this4.selectFeature(layer, new Uint8Array([0, 0, 0, 0])); // toggle select


          layer.setCurrentSelectedId(null);
          _this4.pickColors.select = null;
        }
      }); // unselect normal layer

      var renderList = this.layerService.getRenderList();
      renderList.filter(function (layer) {
        return layer.inited && !layer.isVector && layer.isVisible() && layer.needPick('click');
      }).filter(function (layer) {
        return layer.getCurrentSelectedId() !== null;
      }).map(function (layer) {
        _this4.selectFeature(layer, new Uint8Array([0, 0, 0, 0]));

        layer.setCurrentSelectedId(null);
      });
    }
  }, {
    key: "selectFeature",
    value: function selectFeature(layer, pickedColors) {
      // @ts-ignore
      var _pickedColors = _slicedToArray(pickedColors, 3),
          r = _pickedColors[0],
          g = _pickedColors[1],
          b = _pickedColors[2];

      layer.hooks.beforeSelect.call([r, g, b]);
    }
  }, {
    key: "bindSubLayerEvent",
    value: function bindSubLayerEvent() {
      var _this5 = this;

      /**
       * layer.on('click', (ev) => {}); // 鼠标左键点击图层事件
       * layer.on('mouseenter', (ev) => {}); // 鼠标进入图层要素
       * layer.on('mousemove', (ev) => {}); // 鼠标在图层上移动时触发
       * layer.on('mouseout', (ev) => {}); // 鼠标移出图层要素时触发
       * layer.on('mouseup', (ev) => {}); // 鼠标在图层上单击抬起时触发
       * layer.on('mousedown', (ev) => {}); // 鼠标在图层上单击按下时触发
       * layer.on('contextmenu', (ev) => {}); // 图层要素点击右键菜单
       *
       *  鼠标在图层外的事件
       * layer.on('unclick', (ev) => {}); // 图层外点击
       * layer.on('unmousemove', (ev) => {}); // 图层外移动
       * layer.on('unmouseup', (ev) => {}); // 图层外鼠标抬起
       * layer.on('unmousedown', (ev) => {}); // 图层外单击按下时触发
       * layer.on('uncontextmenu', (ev) => {}); // 图层外点击右键
       * layer.on('unpick', (ev) => {}); // 图层外的操作的所有事件
       */
      this.parent.on('subLayerClick', function (e) {
        _this5.parent.emit('click', _objectSpread({}, e));
      });
      this.parent.on('subLayerMouseMove', function (e) {
        return _this5.parent.emit('mousemove', _objectSpread({}, e));
      });
      this.parent.on('subLayerMouseUp', function (e) {
        return _this5.parent.emit('mouseup', _objectSpread({}, e));
      });
      this.parent.on('subLayerMouseEnter', function (e) {
        return _this5.parent.emit('mouseenter', _objectSpread({}, e));
      });
      this.parent.on('subLayerMouseOut', function (e) {
        return _this5.parent.emit('mouseout', _objectSpread({}, e));
      });
      this.parent.on('subLayerMouseDown', function (e) {
        return _this5.parent.emit('mousedown', _objectSpread({}, e));
      });
      this.parent.on('subLayerContextmenu', function (e) {
        return _this5.parent.emit('contextmenu', _objectSpread({}, e));
      }); // vector layer 图层外事件

      this.parent.on('subLayerUnClick', function (e) {
        return _this5.parent.emit('unclick', _objectSpread({}, e));
      });
      this.parent.on('subLayerUnMouseMove', function (e) {
        return _this5.parent.emit('unmousemove', _objectSpread({}, e));
      });
      this.parent.on('subLayerUnMouseUp', function (e) {
        return _this5.parent.emit('unmouseup', _objectSpread({}, e));
      });
      this.parent.on('subLayerUnMouseDown', function (e) {
        return _this5.parent.emit('unmousedown', _objectSpread({}, e));
      });
      this.parent.on('subLayerUnContextmenu', function (e) {
        return _this5.parent.emit('uncontextmenu', _objectSpread({}, e));
      });
    }
  }, {
    key: "initTileSetManager",
    value: function initTileSetManager() {
      var _this$tilesetManager;

      var source = this.parent.getSource();
      this.tilesetManager = source.tileset;

      if (!this.initedTileset) {
        this.bindTilesetEvent();
        this.initedTileset = true;
      }

      var _this$getCurrentView = this.getCurrentView(),
          latLonBounds = _this$getCurrentView.latLonBounds,
          zoom = _this$getCurrentView.zoom;

      (_this$tilesetManager = this.tilesetManager) === null || _this$tilesetManager === void 0 ? void 0 : _this$tilesetManager.update(zoom, latLonBounds);
    }
  }, {
    key: "mapchange",
    value: function mapchange() {
      var _this$tilesetManager2;

      var _this$getCurrentView2 = this.getCurrentView(),
          latLonBounds = _this$getCurrentView2.latLonBounds,
          zoom = _this$getCurrentView2.zoom;

      if (this.mapService.version === 'GAODE1.x') {
        var _this$parent$getLayer = this.parent.getLayerConfig(),
            visible = _this$parent$getLayer.visible;

        if (zoom < 3 && visible) {
          this.parent.updateLayerConfig({
            visible: false
          });
          this.layerService.reRender();
        } else if (zoom >= 3 && !visible) {
          this.parent.updateLayerConfig({
            visible: true
          });
          this.layerService.reRender();
        }
      }

      if (this.lastViewStates && this.lastViewStates.zoom === zoom && this.lastViewStates.latLonBounds.toString() === latLonBounds.toString()) {
        return;
      }

      this.lastViewStates = {
        zoom: zoom,
        latLonBounds: latLonBounds
      };
      (_this$tilesetManager2 = this.tilesetManager) === null || _this$tilesetManager2 === void 0 ? void 0 : _this$tilesetManager2.throttleUpdate(zoom, latLonBounds);
    }
  }, {
    key: "bindTilesetEvent",
    value: function bindTilesetEvent() {
      var _this6 = this;

      if (!this.tilesetManager) {
        return;
      } // 瓦片数据加载成功
      // eslint-disable-next-line @typescript-eslint/no-unused-vars


      this.tilesetManager.on('tile-loaded', function (tile) {// 将事件抛出，图层上可以监听使用
      }); // 瓦片数据从缓存删除或被执行重新加载

      this.tilesetManager.on('tile-unload', function (tile) {
        // 将事件抛出，图层上可以监听使用
        _this6.tileUnLoad(tile);
      }); // 瓦片数据加载失败
      // eslint-disable-next-line @typescript-eslint/no-unused-vars

      this.tilesetManager.on('tile-error', function (error, tile) {
        // 将事件抛出，图层上可以监听使用
        _this6.tileError(error);
      }); // 瓦片显隐状态更新

      this.tilesetManager.on('tile-update', function () {
        _this6.tileUpdate();
      }); // 地图视野发生改变

      this.mapService.on('zoomend', function () {
        return _this6.viewchange();
      });
      this.mapService.on('moveend', function () {
        return _this6.viewchange();
      });
    } //  防抖操作

  }, {
    key: "getCurrentView",
    value: function getCurrentView() {
      var bounds = this.mapService.getBounds();
      var latLonBounds = [bounds[0][0], bounds[0][1], bounds[1][0], bounds[1][1]];
      var zoom = this.mapService.getZoom();
      return {
        latLonBounds: latLonBounds,
        zoom: zoom
      };
    }
  }, {
    key: "destroy",
    value: function destroy() {
      var _this$tilesetManager3;

      (_this$tilesetManager3 = this.tilesetManager) === null || _this$tilesetManager3 === void 0 ? void 0 : _this$tilesetManager3.destroy();
      this.tileLayerManager.destroy();
    }
  }]);

  return BaseTileLayer;
}();

export { BaseTileLayer as default };