"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getContainerSize = getContainerSize;
exports.getLayerShape = getLayerShape;
exports.getMaskValue = getMaskValue;
exports.isTileChildLoaded = isTileChildLoaded;
exports.isTileLoaded = isTileLoaded;
exports.isTileParentLoaded = isTileParentLoaded;
exports.isVectorTile = isVectorTile;
exports.readPixel = readPixel;
exports.readRasterValue = readRasterValue;
exports.registerLayers = registerLayers;
exports.tileAllLoad = tileAllLoad;
exports.tileVectorParser = void 0;
exports.updateLayersConfig = updateLayersConfig;
exports.updateTileVisible = updateTileVisible;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _l7Core = require("@antv/l7-core");

var _l7Utils = require("@antv/l7-utils");

var tileVectorParser = ['mvt', 'geojsonvt', 'testTile'];
exports.tileVectorParser = tileVectorParser;

function isVectorTile(parserType) {
  return tileVectorParser.indexOf(parserType) >= 0;
}

function registerLayers(parentLayer, layers) {
  layers.map(function (layer) {
    var container = (0, _l7Core.createLayerContainer)(parentLayer.sceneContainer);
    layer.setContainer(container, parentLayer.sceneContainer);
    layer.init();
  });
}

function getLayerShape(layerType, layer) {
  var _layerShape$scale;

  var layerShape = layer.getAttribute('shape');

  if (layerShape && (_layerShape$scale = layerShape.scale) !== null && _layerShape$scale !== void 0 && _layerShape$scale.field) {
    var _layerShape$scale2;

    if (((_layerShape$scale2 = layerShape.scale) === null || _layerShape$scale2 === void 0 ? void 0 : _layerShape$scale2.values) === 'text') {
      return [layerShape.scale.field, layerShape.scale.values];
    }

    return layerShape.scale.field;
  }

  switch (layerType) {
    case 'PolygonLayer':
      return 'fill';

    case 'LineLayer':
      return 'tileline';

    case 'PointLayer':
      return 'circle';

    case 'RasterLayer':
      return 'image';

    default:
      return '';
  }
}

function getMaskValue(layerType, mask) {
  switch (layerType) {
    case 'PolygonLayer':
      return true;

    case 'LineLayer':
      return true;

    case 'PointLayer':
      return false;

    case 'RasterLayer':
      return mask;

    default:
      return mask;
  }
}

function getContainerSize(container) {
  if (container.getContext) {
    return {
      width: container.width / _l7Utils.DOM.DPR,
      height: container.height / _l7Utils.DOM.DPR
    };
  } else {
    return container.getBoundingClientRect();
  }
}

function readRasterValue(tile, mapService, x, y) {
  var _tile$bboxPolygon, _tile$data, _tile$data2, _tile$data3;

  var bbox = (tile === null || tile === void 0 ? void 0 : (_tile$bboxPolygon = tile.bboxPolygon) === null || _tile$bboxPolygon === void 0 ? void 0 : _tile$bboxPolygon.bbox) || [0, 0, 10, -10];

  var _bbox = (0, _slicedToArray2.default)(bbox, 4),
      _bbox$ = _bbox[0],
      minLng = _bbox$ === void 0 ? 0 : _bbox$,
      _bbox$2 = _bbox[1],
      minLat = _bbox$2 === void 0 ? 0 : _bbox$2,
      _bbox$3 = _bbox[2],
      maxLng = _bbox$3 === void 0 ? 10 : _bbox$3,
      _bbox$4 = _bbox[3],
      maxLat = _bbox$4 === void 0 ? -10 : _bbox$4;

  var tileXY = mapService.lngLatToContainer([minLng, minLat]);
  var tileMaxXY = mapService.lngLatToContainer([maxLng, maxLat]);
  var tilePixelWidth = tileMaxXY.x - tileXY.x;
  var tilePixelHeight = tileXY.y - tileMaxXY.y;
  var pos = [(x - tileXY.x) / tilePixelWidth, // x
  (y - tileMaxXY.y) / tilePixelHeight // y
  ];
  var tileWidth = (tile === null || tile === void 0 ? void 0 : (_tile$data = tile.data) === null || _tile$data === void 0 ? void 0 : _tile$data.width) || 1;
  var tileHeight = (tile === null || tile === void 0 ? void 0 : (_tile$data2 = tile.data) === null || _tile$data2 === void 0 ? void 0 : _tile$data2.height) || 1;
  var indexX = Math.floor(pos[0] * tileWidth);
  var indexY = Math.floor(pos[1] * tileHeight);
  var index = Math.max(0, indexY - 1) * tileWidth + indexX;
  var data = tile === null || tile === void 0 ? void 0 : (_tile$data3 = tile.data) === null || _tile$data3 === void 0 ? void 0 : _tile$data3.data[index];
  return data;
}

function readPixel(x, y, rendererService) {
  var readPixels = rendererService.readPixels,
      getContainer = rendererService.getContainer;
  var xInDevicePixel = x * _l7Utils.DOM.DPR;
  var yInDevicePixel = y * _l7Utils.DOM.DPR;

  var _getContainerSize = getContainerSize(getContainer()),
      width = _getContainerSize.width,
      height = _getContainerSize.height;

  width *= _l7Utils.DOM.DPR;
  height *= _l7Utils.DOM.DPR;

  if (xInDevicePixel > width - 1 * _l7Utils.DOM.DPR || xInDevicePixel < 0 || yInDevicePixel > height - 1 * _l7Utils.DOM.DPR || yInDevicePixel < 0) {
    return false;
  }

  var pickedColors = readPixels({
    x: Math.floor(xInDevicePixel),
    // 视口坐标系原点在左上，而 WebGL 在左下，需要翻转 Y 轴
    y: Math.floor(height - (y + 1) * _l7Utils.DOM.DPR),
    width: 1,
    height: 1,
    data: new Uint8Array(1 * 1 * 4)
  });
  return pickedColors;
}

function isTileLoaded(tile) {
  return tile.layerIDList.length === tile.loadedLayers;
}

function isTileChildLoaded(tile) {
  var childs = tile.children;
  return childs.filter(function (child) {
    return isTileLoaded(child);
  }).length === childs.length;
}

function isTileParentLoaded(tile) {
  var parent = tile.parent;

  if (!parent) {
    return true;
  } else {
    return isTileLoaded(parent);
  }
}

function tileAllLoad(tile, callback) {
  var timer = window.setInterval(function () {
    var tileLoaded = isTileLoaded(tile);
    var tileChildLoaded = isTileChildLoaded(tile);
    var tileParentLoaded = isTileParentLoaded(tile);

    if (tileLoaded && tileChildLoaded && tileParentLoaded) {
      callback();
      window.clearInterval(timer);
    }
  }, 36);
}

function updateLayersConfig(layers, key, value) {
  layers.map(function (layer) {
    if (key === 'mask') {
      // Tip: 栅格瓦片生效、设置全局的 mask、瓦片被全局的 mask 影响
      layer.style({
        mask: value
      });
    } else {
      layer.updateLayerConfig((0, _defineProperty2.default)({}, key, value));
    }
  });
}

function dispatchTileVisibleChange(tile, callback) {
  if (tile.isVisible) {
    callback();
  } else {
    tileAllLoad(tile, function () {
      callback();
    });
  }
}

function updateImmediately(layers) {
  var immediately = true;
  layers.map(function (layer) {
    if (layer.type !== 'PointLayer') {
      immediately = false;
    }
  });
  return immediately;
}

function updateTileVisible(tile, layers, layerService) {
  if (layers.length === 0) return;

  if (updateImmediately(layers)) {
    updateLayersConfig(layers, 'visible', tile.isVisible);
    layerService.reRender();
    return;
  }

  dispatchTileVisibleChange(tile, function () {
    updateLayersConfig(layers, 'visible', tile.isVisible);
    layerService.reRender();
  });
}